/*
	parse.c - parse an emkfile

	This file is part of emk.

	emk is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the
	Free Software Foundation, either version 3 of the License, or (at your
	option) any later version.

	emk is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
	for more details.

	You should have received a copy of the GNU General Public License
	along with emk. If not, see <https://www.gnu.org/licenses/>. 
*/

#include <libcolor.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "parse.h"

struct emkfileData {
	char cc;
	char prefix;
};

int checkForRoot() {
	printf(COL_BLUE "INF" COL_RESET " | checking for root access...\n");
	if (geteuid() != 0) {
		printf(COL_BLUE "INF" COL_RESET " | running emk without root\n");
	} else {
		printf(COL_BLUE "INF" COL_RESET " | running emk as root\n");
	}
}

char checkForEmkFile() {
	char cwd[PATH_MAX];
	FILE *emkfile;

	emkfile = fopen(EMKFILE, "r");
	if (emkfile == NULL) {
		printf(COL_RED "ERR" COL_RESET " | emkfile does not exist\n");
		exit(1);
	} else if (getcwd(cwd, sizeof(cwd)) != NULL) {
		printf(COL_BLUE "INF" COL_RESET " | using emkfile at %s/emkfile\n", cwd);
	}
	fclose(emkfile);
}

char readEmkFile() {
	FILE *emkfile;

	char filename[100], contents;

	emkfile = fopen(EMKFILE, "r");
	printf(COL_BLUE "INF" COL_RESET " | opening emkfile\n");
	contents = fgetc(emkfile);
		while (contents != EOF) {
			printf ("%c", contents);
			contents = fgetc(emkfile);
		}
	fclose(emkfile);
}

char includeConfig() {
	char cwd[PATH_MAX];
	FILE *emkconfig, *emkfile, *emkfile_new;
	char filename[100], contents;

	printf(COL_BLUE "INF" COL_RESET " | parsing config.emk\n");
	emkconfig = fopen(EMKCONFIG, "r");
	printf(COL_BLUE "INF" COL_RESET " | parsing emkfile\n");
	emkfile = fopen(EMKFILE, "r");

	printf(COL_BLUE "INF" COL_RESET " | creating new emkfile\n");
	emkfile_new = fopen(EMKFILE_NEW, "a+");

	printf(COL_BLUE "INF" COL_RESET " | running preprocessing tasks...\n");
	/* write emkconfig to new emkfile */
	contents = fgetc(emkconfig);
	while (contents != EOF) {
		fputc(contents, emkfile_new);
		contents = fgetc(emkconfig);
	}

	/* write emkfile to new emkfile */
	contents = fgetc(emkfile);
	while (contents != EOF) {
		fputc(contents, emkfile_new);
		contents = fgetc(emkfile);
	}

	printf(COL_BLUE "INF" COL_RESET " | closing files\n");
	fclose(emkconfig);
	fclose(emkfile);
	fclose(emkfile_new);
}

int main() {
	checkForEmkFile();
	readEmkFile();
	checkForRoot();
	includeConfig();
}
